/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Header name:   c_kpts.h                                           */
/*                                                                   */
/* Abstract:      Defines function prototypes, structures and        */
/*                constants for the z/TPF KPTS driver.               */
/*                                                                   */
/*********************************************************************/

#ifndef __tkpts_
#define __tkpts_

typedef struct kptparms {
            char       kpt;           /* keypoint                   */
            char       ksareq;        /* ksa image number           */
            char       kbareq;        /* kba request                */
            char       klgfbareq;     /* klgfa request              */
            char       hdrsreq;       /* hdrs request               */
            char       refresh;       /* refresh request            */
            short int  error_case;    /* error case                 */
            char       tout;          /* do timeout test            */
            char       cso;           /* do cso cases               */
            char       bso;           /* do bso cases               */
            char       procreq;       /* cross processor req        */
            short int  cpuord;        /* ordinal of cross processor */
            char       cpuid;         /* cpuid of cross processor   */
            char       image;         /* image                      */
            char       snap;          /* snap request               */
            char       verb;          /* verbose                    */
            char       tag;           /* Tagging                    */
            char       evt_name[8];   /* event name                 */
   }
#ifdef __SYSC__
kptparms
#endif
;


const char QKPT_GRAMMAR[] =
          "{KPT-a++ [ERRorcase-d]                                                    \
            [(NO)TOut]                                                               \
            [(NO)VERBose]                                                            \
            [(NO)VERIFY]                                                             \
            [PROC-a]                                                                 \
            [IMage-d]                                                                \
            {[(NO)KSA] | [(NO)KBA] | [(NO)REFRESH] | [(NO)KLGFBA]}                   \
            {[(NO)Cso] | [(NO)Bso]}                                                  \
            [(NO)HDRS]                                                               \
            [(NO)Snap]}";

/*-------------------------------------------------------------------*/
/* make all function extern C to simplify the linkage                */
/*-------------------------------------------------------------------*/

extern "C" {

/*-------------------------------------------------------------------*/
/* macros definition                                                 */
/*-------------------------------------------------------------------*/

#define prterr(x) printf("%s *** Case Failed *** \n", x)
#define prtmsg(x) if ( ecbptr()->ebcm01 == TRUE ) printf(x);  else

/*-------------------------------------------------------------------*/
/* function prototype                                                */
/*-------------------------------------------------------------------*/
boolean getkpt(char *kpt, kptparms *, unsigned char *verbose );
short cpuNameToOrdinal(const char Name);
void cvt_cpuid(char *cpuid_ptr, short *ordinal_ptr);
void getkpta_macro(void);
void getkpta_func(kptparms *);
void getkptb_macro(void);
void getkptb_func(kptparms *);
void getkptc_macro(void);
void getkptc_func(kptparms *);
void getkptd_macro(void);
void getkptd_func(kptparms *);
void getkpte_macro(void);
void getkpte_func(kptparms *);
void getkptf_macro(void);
void getkptf_func(kptparms *);
void getkpti_macro(void);
void getkpti_func(kptparms *);
void getkptm_macro(void);
void getkptm_func(kptparms *);
void getkptv_macro(void);
void getkptv_func(kptparms *);
void getkpt0_macro(void);
void getkpt0_func(kptparms *);
void getkpt1_macro(void);
void getkpt1_func(kptparms *);
void getkpt2_macro(void);
void getkpt2_func(kptparms *);
void getkpt3_macro(void);
void getkpt3_func(kptparms *);
void getkpt4_macro(void);
void getkpt4_func(kptparms *);
void getkpt5_macro(void);
void getkpt5_func(kptparms *);
void getkpt6_macro(void);
void getkpt6_func(kptparms *);
void getkpt9_macro(void);
void getkpt9_func(kptparms *);
void getkptz_macro(void);
void getkptz_func(kptparms *);
};

#endif
