/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  qkp010.cpp                                         */
/*                                                                   */
/* Abstract:      This driver tests the use of the following         */
/*                functions:                                         */
/*                   a       -   getkpta_func()                      */
/*                   b       -   getkptb_func()                      */
/*                   c       -   getkptc_func()                      */
/*                   d       -   getkptd_func()                      */
/*                   e       -   getkpte_func()                      */
/*                   f       -   getkptf_func()                      */
/*                   i       -   getkpti_func()                      */
/*                   m       -   getkptm_func()                      */
/*                   v       -   getkptv_func()                      */
/*                   0       -   getkpt0_func()                      */
/*                   1       -   getkpt1_func()                      */
/*                   2       -   getkpt2_func()                      */
/*                   3       -   getkpt3_func()                      */
/*                   4       -   getkpt4_func()                      */
/*                   5       -   getkpt5_func()                      */
/*                   6       -   getkpt6_func()                      */
/*                   9       -   getkpt9_func()                      */
/*                                                                   */
/* Description:   The driver to run this program is qkp000.cpp.      */
/*********************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/c_eb0eb.h>
#include <tpf/tpfio.h>
#include <unistd.h>
#include <tpf/sysapi.h>
#include <tpf/c_uatbc.h>
#include <tpf/c_ms0ut.h>
#include <c_kpts.h>


static char zeros[] = {'\x00','\x00'};
static char tagval[]={'K','P','T','S'};

#define TAG   TRUE
#define NOTAG FALSE

extern "C"{

int tag_kpt(char *kpt,
            char *cpuid,
            short int *cpuord,
            char *location,
            char *image,
            char *verbose);

#define tag_kpt_location_KSA 0x80
#define tag_kpt_location_KBA 0x40
#define tag_kpt_location_KLGFBA 0x20

void QKP3(struct TPF_regs *regs);
void releaseKPT(void *kp_rec, kptparms *parms_p,int inopts);

/*************************************************/
/*  Keypoint retrieve                            */
/*************************************************/
int retrieveKPT(void **kp_rec, kptparms *parms_p,int inopts, char tag)
{
  int options = inopts;
  char location = 0x00;
  char *tagptr;
  struct snapc_list *snap1[2];
  int rc;
  if (parms_p->ksareq == TRUE )
  {
    if(options==GETKC_HOLD)
      options=0;
    options += GETKC_KSA ;
    location = tag_kpt_location_KSA;
  }
  else if (parms_p->kbareq ==TRUE )
  {
    if(options==GETKC_HOLD)
      options=0;

    options += GETKC_KBA ;
    location = tag_kpt_location_KBA;
  }
  else if (parms_p->klgfbareq ==TRUE )
  {
    if(options==GETKC_HOLD)
      options=0;
    options += GETKC_KLGFBA ;
    location = tag_kpt_location_KLGFBA;
  }
  if (parms_p->procreq == TRUE)
  {
    options += GETKC_PROC;
    if(parms_p->image != NULL )
    {
      options += GETKC_IMAGE;
      rc =tag_kpt(&(parms_p->kpt),
                    &(parms_p->cpuid),
                    &(parms_p->cpuord),
                    &location,
                    &(parms_p->image),
                    &(parms_p->verb));
      if (rc != 0)
      {
        prtmsg("tpf_getkc tag process failed.\n");
        return rc;
      }
      prtmsg("tpf_getkc processing request.\n");

      rc=tpf_getkc(parms_p->kpt,kp_rec,options,(int)parms_p->cpuord,parms_p->image);
      if (rc != 0)
      {
        prtmsg("tpf_getkc test failed.\n");
      }
    }
    else
    {
      if(tag==TAG)
        {
          rc=tag_kpt(&(parms_p->kpt),
                    &(parms_p->cpuid),
                    &(parms_p->cpuord),
                    &location,
                    &(parms_p->image),
                    &(parms_p->verb));
         if (rc != 0)
        {
          prtmsg("tpf_getkc tag process failed.\n");
          return rc;
        }
      }
      prtmsg("tpf_getkc processing request.\n");
      rc=tpf_getkc(parms_p->kpt,kp_rec,options,(int)parms_p->cpuord);
        if (rc != 0)
      {
        prtmsg("tpf_getkc test failed.\n");
      }
      else
      {
        if(tag==TAG)
        {
          tagptr=(char *)(*kp_rec);
          tagptr +=4;
          rc=memcmp(tagptr,tagval,sizeof(tagval));
              if (rc != 0)
          {
            prtmsg("tpf_getkc tag validation failed.\n");
          }
        }
      }
    }
  }
  else if(parms_p->image != NULL )
  {
    options += GETKC_IMAGE;
    if(tag == TAG)
    {
      rc=tag_kpt(&(parms_p->kpt),
                    &(parms_p->cpuid),
                    &(parms_p->cpuord),
                    &location,
                    &(parms_p->image),
                    &(parms_p->verb));
       if (rc != 0)
      {
        prtmsg("tpf_getkc tag process failed.\n");
        return rc;
      }
    }
    prtmsg("tpf_getkc processing request.\n");
    rc=tpf_getkc(parms_p->kpt,kp_rec,options,parms_p->image);
     if (rc != 0)
    {
      prtmsg("tpf_getkc test failed.\n");
    }
    else
    {
      if(tag == TAG)
      {
        tagptr=(char *)(*kp_rec);
        tagptr +=4;
        rc=memcmp(tagptr,tagval,sizeof(tagval));
        if (rc != 0)
        {
          prtmsg("tpf_getkc tag validation failed.\n");
        }
      }
    }
  }
  else
  {
    if(tag == TAG)
    {
      rc=tag_kpt(&(parms_p->kpt),
                    &(parms_p->cpuid),
                    &(parms_p->cpuord),
                    &location,
                    &(parms_p->image),
                    &(parms_p->verb));
       if (rc != 0)
      {
        prtmsg("tpf_getkc tag process failed.\n");
        return rc;
      }
    }
    prtmsg("tpf_getkc processing request.\n");
    rc=tpf_getkc(parms_p->kpt,kp_rec,options);
     if (rc != 0)
      {
      prtmsg("tpf_getkc test failed.\n");
    }
    else
    {
      tagptr=(char *)(*kp_rec);
      tagptr +=4;
      if(tag == TAG)
      {
        rc=memcmp(tagptr,tagval,sizeof(tagval));
            if (rc != 0)
        {
          prtmsg("tpf_getkc tag validation failed.\n");
          if(options & HOLD)
          {
            releaseKPT(*kp_rec, parms_p,inopts);
          }
        }
      }
    }
  }
  if (rc == 0 & parms_p->snap==TRUE)
  {
   snap1[0] = (struct snapc_list *) &ecbptr()->ebw000;
   snap1[1] = (struct snapc_list *) &ecbptr()->ebw016;
   snap1[0]->snapc_len = 100;
   snap1[0]->snapc_name = "KPT-DATA";
   snap1[0]->snapc_tag = *kp_rec;
   snap1[0]->snapc_indir = SNAPC_NOINDIR;
   snap1[1]->snapc_len = 0;
   snapc(SNAPC_RETURN,                   /* return to nsi     */
      0x9111,                            /* snapc id number   */
      NULL,                              /* appended message  */
      snap1,                             /* snapc tag         */
      'A',                               /* snapc prefix      */
      NULL,                              /* display regs      */
      NULL,                              /* use ecb ss and ta */
      NULL);                             /* snapc pgm name    */
  }
  return rc;
}

/*************************************************/
/*  Keypoint update                              */
/*************************************************/
int updtKPT(void *kp_rec, kptparms *parms_p,char inopts,char tag)
{
  void *kp_rec_t;
  int options = inopts;
  char *tagptr;
  int rc;
  if(parms_p->ksareq ==TRUE)
  {
    options += UPDKC_KSA;
  }
  else if(parms_p->kbareq==TRUE )
  {
    options += UPDKC_KBA;
  }
  else if(parms_p->klgfbareq==TRUE )
  {
    options += UPDKC_KLGFBA;
  }
  if (parms_p->procreq == TRUE)
  {
    options +=UPDKC_PROC;
    if(parms_p->image != NULL)
    {
      options +=UPDKC_IMAGE;
      options += UPDKC_NOWAIT;
      prtmsg("tpf_updkc NOWAIT processing request.\n");
      rc= tpf_updkc(parms_p->kpt,kp_rec,options,(int)parms_p->cpuord,parms_p->image);
      if(waitc())
      {
        prtmsg("tpf_updkc test failed.\n");
        return rc;
      }
      options -= UPDKC_NOWAIT;
      prtmsg("tpf_updkc WAIT processing request.\n");
      if(rc==tpf_updkc(parms_p->kpt,kp_rec,options,(int)parms_p->cpuord,parms_p->image))
      {
        prtmsg("tpf_updkc test failed.\n");
      }
      prtmsg("tpf_getkc after update processing request.\n");
      if(rc==tpf_getkc(parms_p->kpt,&kp_rec_t,options,(int)parms_p->cpuord,parms_p->image))
      {
        prtmsg("tpf_getkc after update test. failed.\n");
        return rc;
      }
      if(tag == TAG)
      {
        tagptr=(char *)(kp_rec_t);
        tagptr +=4;
        if(rc==memcmp(tagptr,tagval,sizeof(tagval)))
        {
          prtmsg("tpf_getkc tag validated.\n");
          rc=0;
        }
        else
        {
          prtmsg("tpf_getkc tag validation failed.\n");
          rc=-1;
        }
      }
    }
    else
    {
      prtmsg("tpf_updkc PROC test.\n");

      options += UPDKC_NOWAIT;
      prtmsg("tpf_updkc NOWAIT processing request.\n");
      rc= tpf_updkc(parms_p->kpt,kp_rec,options,(int)parms_p->cpuord);
      if(waitc())
      {
        prtmsg("tpf_updkc test failed.\n");
        return rc;
      }
      options -= UPDKC_NOWAIT;
      prtmsg("tpf_updkc WAIT processing request.\n");
      if(rc==tpf_updkc(parms_p->kpt,kp_rec,options,(int)parms_p->cpuord))
      {
        prtmsg("tpf_updkc test failed.\n");
      }
      options=inopts;
      options +=GETKC_PROC;

      prtmsg("tpf_getkc after update processing request.\n");
      if(rc==tpf_getkc(parms_p->kpt,&kp_rec_t,options,(int)parms_p->cpuord))
      {
        prtmsg("tpf_getkc after update test. failed.\n");
        return rc;
      }
      if(tag == TAG)
      {
        tagptr=(char *)(kp_rec_t);
        tagptr +=4;
        if(rc==memcmp(tagptr,tagval,sizeof(tagval)))
        {
          prtmsg("tpf_getkc tag validated.\n");
          rc=0;
        }
        else
        {
          prtmsg("tpf_getkc tag validation failed.\n");
          rc=-1;
        }
      }
    }
  }

  else if(parms_p->image != NULL)
  {
    options += UPDKC_IMAGE;
    if(parms_p->ksareq==TRUE)
    {
      options += UPDKC_KSA;
    }
    else if(parms_p->kbareq==TRUE )
    {
      options += UPDKC_KBA;
    }

    options += UPDKC_NOWAIT;
    prtmsg("tpf_updkc NOWAIT processing request.\n");
    rc= tpf_updkc(parms_p->kpt,kp_rec,options,parms_p->image);
    if(waitc())
    {
      prtmsg("tpf_updkc test failed.\n");
      return rc;
    }
    options -= UPDKC_NOWAIT;
    prtmsg("tpf_updkc WAIT processing request.\n");
    if(rc==tpf_updkc(parms_p->kpt,kp_rec,options,parms_p->image))
    {
      prtmsg("tpf_updkc test failed.\n");
      return rc;
    }
    prtmsg("tpf_getkc after update processing request.\n");
    if(rc==tpf_getkc(parms_p->kpt,&kp_rec_t,options,parms_p->image))
    {
      prtmsg("tpf_getkc after update test failed.\n");
      return rc;
    }
  }
  else
  {
    options += UPDKC_NOWAIT;
    prtmsg("tpf_updkc NOWAIT processing request.\n");
    rc= tpf_updkc(parms_p->kpt,kp_rec,options);
    if(waitc())
    {
      prtmsg("tpf_updkc NOWAIT test failed.\n");
      return rc;
    }
    options -= UPDKC_NOWAIT;
    prtmsg("tpf_updkc WAIT processing request.\n");
    if(rc==tpf_updkc(parms_p->kpt,kp_rec,options))
    {
      prtmsg("tpf_updkc test failed.\n");
      return rc;
    }

    prtmsg("tpf_getkc after update processing request.\n");
    if(rc==tpf_getkc(parms_p->kpt,&kp_rec_t,options))
    {
      prtmsg("tpf_getkc after update test failed.\n");
      return rc;
    }
  }
  tagptr=(char *)(kp_rec_t);
  tagptr +=4;
  if(tag == TAG)
  {
    if(rc==memcmp(tagptr,tagval,sizeof(tagval)))
    {
      prtmsg("tpf_getkc tag validated.\n");
      rc=0;
    }
    else
    {
      prtmsg("tpf_getkc tag validation failed.\n");
      rc=-1;
    }
  }
  free(kp_rec_t);
  return rc;
}

/*************************************************/
/*  Keypoint release                             */
/*************************************************/
void releaseKPT(void *kp_rec, kptparms *parms_p,int inopts)
{
  int options = inopts;
  if(parms_p->kbareq==TRUE | parms_p->ksareq==TRUE | parms_p->klgfbareq==TRUE)
  {
    free(kp_rec);
    return;
  }

  prtmsg("tpf_relkc processing request.\n");
  if (parms_p->procreq == TRUE )
  {
    options += RELKC_PROC ;
    tpf_relkc(parms_p->kpt,options,(int)parms_p->cpuord);
    free(kp_rec);
  }
  else
  {
    tpf_relkc(parms_p->kpt,options);
    free(kp_rec);
  }
  return ;
}


/*************************************************/
/*  Keypoint timeout                             */
/*************************************************/

void timeoutKPT(kptparms *parms_p)
{
  char          buffer[80];            /* used to print out messages */
  int rc;
  struct ev0bk event_blk;
  if(parms_p->ksareq == TRUE | parms_p->kbareq == TRUE | parms_p->ksareq == TRUE)
  {
      prtmsg("kpts: Timeout not supported for KBA/KSA/KLGFBA.\n");
      return;
  }
  prtmsg("kpts: Timeout error test.\n");
  event_blk.evnpstinf.evnbkc1 = 1;
  evntc(&event_blk,EVENT_CNT,'N',10,EVNTC_1052);
  memcpy(parms_p->evt_name,event_blk.evnbkn,sizeof(event_blk.evnbkn));
  parms_p->verb=(char)ecbptr()->ebcm01;
  cremc(sizeof(kptparms),(char *)parms_p,QKP1);
  sprintf(buffer,"getkc: Waiting for timeout event kpt-%c.\n",parms_p->kpt);
      prtmsg(buffer);
  rc=evnwc(&event_blk,EVENT_CNT);
  if(rc!=0)
  {
    prtmsg("kpts: Event wait error.\n");
  }
  return;
}
void eat_malloc()
{
  prtmsg("kpts: attempting malloc error.\n");
  void *tptr = NULL;
  int  msize = 1000000;
  tptr=malloc(msize);
  while (msize > 1055)
  {
    tptr = malloc(msize);
    if(tptr==NULL)
    {
      msize = msize/2;
    }
  }
  return;
}

/*********************************************************************/
/*  get keypoint A test                                              */
/*********************************************************************/
void getkpta_func(kptparms *parms_p)
{
  parms_p->kpt='A';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = GETKC_NONE;
  prtmsg("kpts: testing kpt A\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */

  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
      else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-A.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-A retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-A failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-A HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-A retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-A HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/

  prtmsg("getkc: retrieving kpt-A HOLD.\n");
  options=GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-A retrieved\n");
     prtmsg(buffer);
     options=NULL;

     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-A updated\n");
       prtmsg(buffer);
     }
     else
     {
       prterr("tpf_updkc: kpt-A update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-A released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-A failed\n");
  }
  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt A for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt A for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt A for timeout test.\n");
    }
  }
  prtmsg("kpts: completed test kpt A.\n");
  return;
} /*end of getkpta_func()                                             */

/*********************************************************************/
/*  get keypoint B test                                              */
/*********************************************************************/
void getkptb_func(kptparms *parms_p)
{
  parms_p->kpt='B';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt B\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */

  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }

  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-B.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-B retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-B failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-B REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-B retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-B failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-B HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-B retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-B HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-B HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-B retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-B updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-B update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-B released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-B failed\n");
  }
  prtmsg("kpts: completed test kpt B.\n");

  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt B for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt B for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt B for timeout test.\n");
    }
  }

  return;
} /*end of getkptb_func()                                             */

/*********************************************************************/
/*  get keypoint C test                                              */
/*********************************************************************/
void getkptc_func(kptparms *parms_p)
{
  parms_p->kpt='C';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt C\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-C.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-C retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-C failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-C REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-C retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-C failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-C HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-C retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-C HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-C HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-C retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-C updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-C update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-C released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-C failed\n");
  }
  prtmsg("kpts: completed test kpt C.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt C for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt C for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt C for timeout test.\n");
    }
  }

  return;
} /*end of getkptc_func()                                             */

/*********************************************************************/
/*  get keypoint D test                                              */
/*********************************************************************/
void getkptd_func(kptparms *parms_p)
{
  parms_p->kpt='D';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt D\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-D.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-D retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-D failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-D REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-D retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-D failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-D HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-D retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-D HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-D HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-D retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-D updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-D update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-D released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-D failed\n");
  }
  prtmsg("kpts: completed test kpt D.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt D for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt D for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt D for timeout test.\n");
    }
  }

  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
  if (parms_p->error_case == TRUE)
  {
    prtmsg("kpts: kpt-D testing errors.\n");
    eat_malloc();

    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if (rc == GETKC_MALOC)
    {
      prtmsg("kpts: successful malloc error test.\n");
    }
    else if (rc == 0)
         {
           prtmsg("kpts: Failed malloc error test.\n");
         }
         else
         {
           prtmsg("kpts: Malloc unplanned error.\n");
         }

    prtmsg("kpts: kpt-D Error tests completed.\n");
  }
  return;
} /*end of getkptd_func()                                             */

/*********************************************************************/
/*  get keypoint E test                                              */
/*********************************************************************/
void getkpte_func(kptparms *parms_p)
{
  parms_p->kpt='E';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt E\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-E.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-E retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-E failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-E HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-E retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-E HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-E HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-E retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-E updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-E update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-E released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-E failed\n");
  }
  prtmsg("kpts: completed test kpt E.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt E for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt E for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt E for timeout test.\n");
    }
  }

  return;
} /*end of getkpte_func()                                             */

/*********************************************************************/
/*  get keypoint I test                                              */
/*********************************************************************/
void getkpti_func(kptparms *parms_p)
{
  parms_p->kpt='I';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt I\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-I.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-I retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-I failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-I REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-I retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-I failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-I HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-I retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-I HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-I HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-I retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-I updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-I update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-I released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-I failed\n");
  }
  prtmsg("kpts: completed test kpt I.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt I for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt I for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt I for timeout test.\n");
    }
  }
  return;
} /*end of getkpti_func()                                             */

/*********************************************************************/
/*  get keypoint M test                                              */
/*********************************************************************/
void getkptm_func(kptparms *parms_p)
{
  parms_p->kpt='M';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt M\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-M.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-M retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-M failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-m REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-M retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-M failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-M HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-M retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-M HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-M HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-M retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-M updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-M update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-M released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-M failed\n");
  }
  prtmsg("kpts: completed test kpt M.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt M for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt M for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt M for timeout test.\n");
    }
  }

  return;
} /*end of getkptm_func()                                             */

/*********************************************************************/
/*  get keypoint V test                                              */
/*********************************************************************/
void getkptv_func(kptparms *parms_p)
{
  parms_p->kpt='V';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt V\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-V.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-V retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-V failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-V HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-V retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-V HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-V HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-V retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-V updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-V update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-V released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-V failed\n");
  }
  prtmsg("kpts: completed test kpt V.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt V for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt V for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt V for timeout test.\n");
    }
  }

  return;
} /*end of getkptv_func()                                             */

/*********************************************************************/
/*  get keypoint 0 test                                              */
/*********************************************************************/
void getkpt0_func(kptparms *parms_p)
{
  parms_p->kpt='0';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 0\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");
/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-0.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-0 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-0 failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-0 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-0 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-0 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-0 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-0 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-0 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-0 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-0 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-0 failed\n");
  }
  prtmsg("kpts: completed test kpt 0.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 0 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 0 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 0 for timeout test.\n");
    }
  }

} /*end of getkpt0_func()                                             */

/*********************************************************************/
/*  get keypoint 1 test                                              */
/*********************************************************************/
void getkpt1_func(kptparms *parms_p)
{
  parms_p->kpt='1';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 1\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");
/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-1.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-1 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-1 failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-1 REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-1 retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-1 failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-1 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-1 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-1 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-1 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-1 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-1 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-1 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-1 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-1 failed\n");
  }
  prtmsg("kpts: completed test kpt 1.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 1 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 1 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 1 for timeout test.\n");
    }
  }

  return;
} /*end of getkpt1_func()                                             */

/*********************************************************************/
/*  get keypoint 2 test                                              */
/*********************************************************************/
void getkpt2_func(kptparms *parms_p)
{
  parms_p->kpt='2';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 2\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-2.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-2 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-2 failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-2 REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-2 retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-2 failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-2 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-2 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-2 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-2 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-2 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-2 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-2 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-2 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-2 failed\n");
  }
  prtmsg("kpts: completed test kpt 2.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 2 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 2 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 2 for timeout test.\n");
    }
  }

  return;
} /*end of getkpt2_func()                                             */

/*********************************************************************/
/*  get keypoint 3 test                                              */
/*********************************************************************/
void getkpt3_func(kptparms *parms_p)
{
  parms_p->kpt='3';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 3\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-3.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-3 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-3 failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-3 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-3 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-3 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-3 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-3 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-3 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-3 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-3 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-3 failed\n");
  }
  prtmsg("kpts: completed test kpt 3.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 3 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 3 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 3 for timeout test.\n");
    }
  }

  return;
} /*end of getkpt3_func()                                             */

/*********************************************************************/
/*  get keypoint 4 test                                              */
/*********************************************************************/
void getkpt4_func(kptparms *parms_p)
{
  parms_p->kpt='4';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 4\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");
/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-4.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-4 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-4 failed\n");
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-4 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-4 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-4 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-4 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-4 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-4 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-4 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-4 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-4 failed\n");
  }
  prtmsg("kpts: completed test kpt 4.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 4 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 4 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 4 for timeout test.\n");
    }
  }

  return;
} /*end of getkpt4_func()                                             */

/*********************************************************************/
/*  get keypoint 5 test                                              */
/*********************************************************************/
void getkpt5_func(kptparms *parms_p)
{
  parms_p->kpt='5';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 5\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");
/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-5.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-5 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-5 failed\n");
  }


/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-5 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-5 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-5 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-5 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-5 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-5 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-5 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-5 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-5 failed\n");
  }
  prtmsg("kpts: completed test kpt 5.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 5 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 5 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 5 for timeout test.\n");
    }
  }

} /*end of getkpt5_func()                                             */

/*********************************************************************/
/*  get keypoint 6 test                                              */
/*********************************************************************/
void getkpt6_func(kptparms *parms_p)
{
  parms_p->kpt='6';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 6\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");

/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-6.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-6 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-6 failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-6 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-6 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-6 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-6 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-6 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-6 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-6 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-6 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-6 failed\n");
  }
  prtmsg("kpts: completed test kpt 6.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 6 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 6 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 6 for timeout test.\n");
    }
  }

  return;
} /*end of getkpt6_func()                                             */

/*********************************************************************/
/*  get keypoint 9 test                                              */
/*********************************************************************/
void getkpt9_func(kptparms *parms_p)
{
  parms_p->kpt='9';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt 9\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");
/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-9.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-9 retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-9 failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-9 REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-9 retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-9 failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-9 HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-9 retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-9 HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-9 HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-9 retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,parms_p->tag);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-9 updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-9 update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-9 released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-9 failed\n");
  }
  prtmsg("kpts: completed test kpt 9.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt 9 for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,parms_p->tag);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt 9 for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt 9 for timeout test.\n");
    }
  }

  return;
} /*end of getkpt9_func()                                             */



/*********************************************************************/
/*  get keypoint Z test                                              */
/*********************************************************************/
void getkptz_func(kptparms *parms_p)
{
  parms_p->kpt='Z';
  void *kp_ptr = NULL;                 /* pointer to keypoint record */
  int options = 0;
  prtmsg("kpts: testing kpt Z\n");
  int rc = 0;                          /* return code                */
  char          buffer[80];            /* used to print out messages */
  if (parms_p->error_case != 0)
  {
  /*************************************************************/
  /* Error tests                                               */
  /*************************************************************/
    if (parms_p->error_case == 1)
    {
      sprintf(buffer,"kpts: kpt-%c testing error case 1.\n",parms_p->kpt);
      prtmsg(buffer);

      eat_malloc();
      options=NULL;
      rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
      if (rc == GETKC_MALOC)
      {
        prtmsg("kpts: successful malloc error test.\n");
      }
      else if (rc == 0)
        {
          prtmsg("kpts: Failed malloc error test.\n");
        }
        else
        {
          prtmsg("kpts: Malloc unplanned error.\n");
        }
      sprintf(buffer,"kpts: kpt-%c Error case 1 completed.\n",parms_p->kpt);
      prtmsg(buffer);
    }
    else if (parms_p->error_case == 2)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 2.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        eat_malloc();
        options=NULL;
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        if (rc == UPDKC_MALOC)
        {
          prtmsg("kpts: successful malloc error test.\n");
          options=NULL;
          releaseKPT(kp_ptr,parms_p,options);

        }
        else if (rc == 0)
          {
            options=NULL;
            releaseKPT(kp_ptr,parms_p,options);
            prtmsg("kpts: Failed malloc error test.\n");
          }
          else
          {
            prtmsg("kpts: Malloc unplanned error.\n");
          }
        sprintf(buffer,"kpts: kpt-%c Error case 2 completed.\n",parms_p->kpt);
        prtmsg(buffer);

      }
    else if (parms_p->error_case == 3)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 3.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Exit while holding. Expecting CTL-8\n",parms_p->kpt);
        prtmsg(buffer);
      }
    else if (parms_p->error_case == 4)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 4.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Dump exit hold while holding. Expecting OPR-F00F00\n",parms_p->kpt);
        prtmsg(buffer);
        serrc_op((enum t_serrc)(SERRC_EXIT+SERRC_NO_HEAP),0xf00f00,"Case 4 test dump.",NULL);

      }
    else if (parms_p->error_case == 5)
      {
        sprintf(buffer,"kpts: kpt-%c testing error case 5.\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=UPDKC_UNHOLD;
        sprintf(buffer,"kpts: kpt-%c UNHOLD-not holding. CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 6)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 6-Invalid Image. Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_KSA;
        parms_p->image=0xff;              /* Invalid image */
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 7)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 7-Corrupt HDRS.\n",parms_p->kpt);
        prtmsg(buffer);
        options=GETKC_HDRS+GETKC_HOLD;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        memcpy(kp_ptr,zeros,sizeof(zeros));
        options=UPDKC_UNHOLD+UPDKC_HDRS;
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640F2\n",parms_p->kpt);
        prtmsg(buffer);
        rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
      }
    else if (parms_p->error_case == 8)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 8-RELKC-not holding.\n",parms_p->kpt);
        prtmsg(buffer);
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        options=NULL;
        sprintf(buffer,"kpts: kpt-%c Expecting CTL-24\n",parms_p->kpt);
        prtmsg(buffer);
        releaseKPT(kp_ptr,parms_p,options);
      }
    else if (parms_p->error_case == 9)
      {
        sprintf(buffer,"kpts: kpt-%c Error case 9-Invalid PROC. \n",parms_p->kpt);
        prtmsg(buffer);
        sprintf(buffer,"kpts: kpt-%c Expecting OPR-0640FD\n",parms_p->kpt);
        prtmsg(buffer);
        options=NULL;
        parms_p->cpuord=254;
        parms_p->procreq=TRUE;
        rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
        prtmsg("tpf_getkc Invalid PROC test failed.\n");
      }
    return;
  }
  prtmsg("getkc: entering getkc_func.\n");
/*****************************************************************/
/* Retrieve the keypoint no parms                                */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-Z.\n");
  options = GETKC_NONE;
  rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
  if(rc==0)
  {
    sprintf(buffer, "tpf_getkc: kpt-Z retrieved \n");
    prtmsg(buffer);
    free(kp_ptr);
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-Z failed\n");
  }

/*****************************************************************/
/* Retrieve the keypoint REFRESH                                 */
/*****************************************************************/
  if(parms_p->refresh ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-Z REFRESH.\n");
    options = GETKC_REFRESH;
    rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-Z retrieved REFRESH \n");
      prtmsg(buffer);
      free(kp_ptr);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-Z failed REFRESH\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint HDRS                                    */
/*****************************************************************/
  if(parms_p->hdrsreq ==TRUE)
  {
    prtmsg("getkc: retrieving kpt-Z HDRS.\n");
    options = GETKC_HDRS;
    rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
    if(rc==0)
    {
      sprintf(buffer, "tpf_getkc: kpt-Z retrieved HDRS\n");
      prtmsg(buffer);
    }
    else
    {
      sprintf(buffer, "tpf_getkc: kpt-Z HDRS failed\n");
   }
  }
/*****************************************************************/
/* Retrieve the keypoint with hold                               */
/*****************************************************************/
  prtmsg("getkc: retrieving kpt-Z HOLD.\n");
  options = GETKC_HOLD;
  rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
  if(rc==0)
  {
     sprintf(buffer, "tpf_getkc: kpt-Z retrieved\n");
     prtmsg(buffer);

     options=NULL;
     rc=updtKPT(kp_ptr,parms_p,options,NOTAG);
     if(rc==0)
     {
       sprintf(buffer, "tpf_updkc: kpt-Z updated\n");
       prtmsg(buffer);
     }
     else
     {
      prterr("tpf_updkc: kpt-Z update failed.");
     }
     /*************************************************************/
     /* tpf_relkc - Release keypoint                              */
     /*************************************************************/
     options=NULL;
     releaseKPT(kp_ptr,parms_p,options);
     prtmsg("tpf_relkc: kpt-Z released.\n");
  }
  else
  {
    sprintf(buffer, "tpf_getkc: kpt-Z failed\n");
  }
  prtmsg("kpts: completed test kpt Z.\n");


  /*********************************************************/
  /* if TOUT is specified...                               */
  /*********************************************************/
  if (parms_p->tout == TRUE)
  {
    options= GETKC_HOLD;
    prtmsg("getkc: ready to hold kpt Z for timeout test.\n");
    rc=retrieveKPT(&kp_ptr,parms_p,options,NOTAG);
    if(rc==0)
    {
      timeoutKPT(parms_p);
      options=NULL;
      releaseKPT(kp_ptr,parms_p,options);
      prtmsg("getkc: released kpt Z for timeout test.\n");
    }
    else
    {
      prtmsg("getkc: Error holding kpt Z for timeout test.\n");
    }
  }

  return;
} /*end of getkptZ_func()                                             */






int tag_kpt(char *kpt,
            char *cpuid,
            short int *cpuord,
            char *location,
            char *image,
            char *verbose)
{
  struct TPF_regs regs;
  char          buffer[80];            /* used to print out messages */

    char *desc = "";
    if(*location & tag_kpt_location_KSA)
      desc= "KSA";
    else if(*location & tag_kpt_location_KBA)
      desc="KBA";
    else if(*location & tag_kpt_location_KLGFBA)
      desc="KLGFBA";
    sprintf(buffer,"Tagging kpt-%c cpu-%c,image-%d,area-%s.\n",*kpt,*cpuid,*image,desc);
    prtmsg(buffer);

    memcpy(&(ecbptr()->ebw000),tagval,sizeof(tagval));
    ecbptr()->ebw004=*kpt;
    ecbptr()->ebw006=*cpuid;
    memcpy(&(ecbptr()->ebw007),cpuord,sizeof(cpuord));
    ecbptr()->ebw009=*location;
    ecbptr()->ebw010=*image;
    ecbptr()->ebw011=*verbose;
    QKP3(&regs);
    return(int(ecbptr()->ebw036));
}

}/*end of extern "C"                                                 */
