/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  qkp100.cpp                                         */
/* Module         QKP1                                               */
/*                                                                   */
/* Abstract:      Part of the KPTS Driver package                    */
/*                                                                   */
/* Description:   This segment will test the TIMEOUT option of the   */
/*                tpf_getkc() API for the KPTS driver and post an    */
/*                event back to the parent.                          */
/*                                                                   */
/* ECB fields                                                        */
/* ebw000(8)  pointer to struct kptparms                             */
/* ebcm01(1)  verbose indicator                                      */
/*                                                                   */
/*********************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <tpf/tpfapi.h>
#include <tpf/tpfio.h>

#include <c_kpts.h>

/*-------------------------------------------------------------------*/
/* make all function extern C to simplify the linkage                */
/*-------------------------------------------------------------------*/

extern "C" {

/*********************************************************************/
/*                                                                   */
/* QKP1( )                                                           */
/*                                                                   */
/* Entry point for the timeout test. Depending on the settings passed*/
/* in the struct kptparms (from the input msg) a HOLD will be        */
/* attempted for the requested keypoint with the same options that   */
/* were used by the parent ECB to hold the kpt. If successful, the   */
/* request will timeout and display a successful status before       */
/* posting a completion event back to the parent.                    */
/*                                                                   */
/*********************************************************************/
void QKP1()
{
  ecbptr()->ebrout = 0x010000;
  kptparms *parms_p;
  parms_p = (kptparms *)&ecbptr()->ebw000;
  ecbptr()->ebcm01=parms_p->verb;
  prtmsg("kpts: Timeout test starting.\n");
  struct ev0bk *pb;
  void *kp_rec;
  int rc;
  int timeout = 2;
  int options = GETKC_HOLD+GETKC_TIMEOUT;
  pb = (ev0bk *)malloc(sizeof(ev0bk));
  memcpy(pb->evnbkn,parms_p->evt_name,sizeof(pb->evnbkn));

  if (parms_p->procreq == TRUE)
  {
    options += GETKC_PROC;
    if (parms_p->image != NULL)
    {
      options +=GETKC_IMAGE;
      if(parms_p->ksareq == TRUE)
      {
        options += GETKC_KSA;
        prtmsg("kpts: Issuing getkc for PROC IMAGE KSA TIMEOUT test.\n");
        rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->cpuord,parms_p->image,timeout);
      }
      else
      {
        if (parms_p->kbareq==TRUE)
        {
          options += GETKC_KBA;
          prtmsg("kpts: Issuing getkc PROC IMAGE KBA TIMEOUT test.\n");
          rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->cpuord,parms_p->image,timeout);
        }
        else
        {
          prtmsg("kpts: Issuing getkc PROC IMAGE TIMEOUT test.\n");
          rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->cpuord,parms_p->image,timeout);
        }
      }
    }
    else
    {
      if(parms_p->ksareq == TRUE)
      {
        options += GETKC_KSA;
        prtmsg("kpts: Issuing getkc for PROC KSA TIMEOUT test.\n");
        rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->cpuord,timeout);
      }
      else
      {
        if (parms_p->kbareq ==TRUE)
        {
          options += GETKC_KBA;
          prtmsg("kpts: Issuing getkc PROC KBA TIMEOUT test.\n");
          rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->cpuord,timeout);
        }
        else
        {
          prtmsg("kpts: Issuing getkc PROC TIMEOUT test.\n");
          rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->cpuord,timeout);
        }
      }
    }
  }
  else
  {
    if(parms_p->image != NULL)
    {
      options +=GETKC_IMAGE;
      if(parms_p->ksareq == TRUE)
      {
        options += GETKC_KSA;
        prtmsg("kpts: Issuing getkc for IMAGE KSA TIMEOUT test.\n");
        rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->image,timeout);
      }
      else
      {
        if (parms_p->kbareq==TRUE)
        {
          options += GETKC_KBA;
          prtmsg("kpts: Issuing getkc IMAGE KBA TIMEOUT test.\n");
          rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->image,timeout);
        }
        else
        {
          prtmsg("kpts: Issuing getkc IMAGE TIMEOUT test.\n");
          rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->image,timeout);
        }
      }
    }
    else
    {
      if (parms_p->ksareq == TRUE)
      {
        options += GETKC_KSA;
        prtmsg("kpts: Issuing getkc KSA TIMEOUT test.\n");
        rc= tpf_getkc(parms_p->kpt,&kp_rec,options,timeout);
      }
      else
      {
        if (parms_p->kbareq ==TRUE)
        {
          options += GETKC_KBA;
          prtmsg("kpts: Issuing getkc KBA TIMEOUT test.\n");
          rc= tpf_getkc(parms_p->kpt,&kp_rec,options,timeout);
        }
        else
        {
          if (parms_p->klgfbareq ==TRUE)
          {
            options += GETKC_KLGFBA;
            prtmsg("kpts: Issuing getkc PROC KLGFBA TIMEOUT test.\n");
            rc= tpf_getkc(parms_p->kpt,&kp_rec,options,parms_p->cpuord,timeout);
          }
          else
          {
            prtmsg("kpts: Issuing getkc TIMEOUT keypoint.\n");
            rc= tpf_getkc(parms_p->kpt,&kp_rec,options,timeout);
          }
        }
      }
    }
  }
  if(rc==GETKC_TIME)
  {
    prtmsg("kpts: Timeout test successful.\n");
  }
  else
  {
    prtmsg("kpts: Timeout test failed to timeout.\n");
    if(rc==0)
    {
      if (parms_p->procreq==TRUE )
      {
       options += RELKC_PROC;
       tpf_relkc(parms_p->kpt,options,parms_p->cpuord);
       free((void **)kp_rec);
      }
      else
      {
       tpf_relkc(parms_p->kpt,options);
       free((void **)kp_rec);
      }
    }
  }
  rc = postc(pb,EVENT_CNT,0);
  if(rc != 0)
  {
    prtmsg("kpts: Timeout post malfunction.\n");
  }
  else
  {
    if (parms_p->procreq==TRUE )
    {
      options += RELKC_PROC;
      tpf_relkc(parms_p->kpt,options,parms_p->cpuord);
    }
    else
    {
      tpf_relkc(parms_p->kpt,options);
    }
  }
  exit(0);
}


} /* end of extern "C"                                               */
